/******************************************************************************
FILE:				BoTimer.h
LANGUAGE:			Objective-C
SYSTEM:				ANSI (stdio)
USER-INTERFACE:		None
DESCRIPTION
	This class defines objects who send to themselves the "behave" message 
	after a delay has elapsed or periodically.
	The precision of the period may be less than the millisecond.
AUTHORS
	<PJB> Pascal J. Bourguignon
MODIFICATIONS
	1992/02/03 <PJB> Creation. (PeriodicTask)
	1992/08/16 <PJB> Renamed BoTimer and added the one-shot functionnality.
						Augmented the parameter precision to 1 micro-second.
	1993/01/17 <PJB> Converted from C++ to Objective-C.
	1993/03/19 <PJB> Updated to use newly Bc- prefixed interfaces.
LEGAL
	Copyright Pascal J. Bourguignon 1992 - 1993
	All rights reserved.
	This program may not be included in any commercial product without the 
	author written permission. It may be used freely for any non-commercial 
	purpose, provided that this header is always included.
******************************************************************************/
#import <BcInterface.h>
#import <BcTypes.h>
#import <BoObject.h>

	
	typedef struct {
		CARD32			seconds;
		CARD32			microseconds;
	} 				BoTimer_TimeT;

@interface BoTimer:BoObject
{
	id						delegate;
	void*					opaque;
}

// birth and death:

	-(id)init;
		/*
			DO:			initialize self.
			RETURN:		self.
			NOTE:		[[BoTimer alloc] init];
		*/

	-(id)free;
		/*
			DO:			free self.
			RETURN:		NIL.
		*/

// override BoObject methods:

	-(void)printOn:(FILE*)file;
		/*
			DO:			dumps self on the file.
		*/
			
// new methods:

	-(CARD32)precision;
		/*
			RETURN:		the precision in microseconds of the period.
			NOTE:		this may be greater than 1e6 microseconds.
		*/

	-(void)startPhase:(BoTimer_TimeT*)phase interval:(BoTimer_TimeT*)interval;
		/*
			DO:			sends to itself the behave message at instants:
							t_0+phase+(k*interval) if interval#0, 
						or once at t_0+phase is interval=0.
		*/

	-(void)stop;
		/*
			DO:			stop sending the behave message.
		*/

	-(void)behave;
		/*
			DO:			whatever the concrete subclasses want to do 
						periodically.
			WARNING:	behave is called at interrupt time.
			NOTE:		sends a behave message to the delegate 
						by [delegate behave].
		*/
		
	-(id)delegate:(id)newdelegate;
		/*
			NOTE:		the delegate is send a behave message by 
						[self behave] (at interrupt time!)
			DO:			set the delegate.
			RETURN:		the old delegate.
		*/
		
	-(id)delegate;
		/*
			RETURN:		the current delegate.
		*/
	
@end //BoTimer.
